@@ -5,7 +5,10 @@ use Data::Printer::Filter;
 use Data::Printer::Common;
 use Scalar::Util ();
 
-filter 'ARRAY' => sub {
+filter 'ARRAY' => \&parse;
+
+
+sub parse {
     my ($array_ref, $ddp) = @_;
 
     my $tied = '';
@@ -55,8 +58,13 @@ filter 'ARRAY' => sub {
             next;
         }
 
+        my $original_varname = $ddp->current_name;
         # if name was "var" it must become "var[0]", "var[1]", etc
-        $ddp->current_name( $ddp->current_name . "[$idx]" );
+        $ddp->current_name(
+            $original_varname
+            . ($ddp->arrows eq 'all' || ($ddp->arrows eq 'first' && $ddp->current_depth == 1) ? '->' : '')
+            . "[$idx]"
+        );
 
         if ($has_index) {
             substr($string, -$local_padding) = ''; # get rid of local padding
@@ -71,7 +79,7 @@ filter 'ARRAY' => sub {
         my $ref = ref $array_ref->[$idx];
         if ($ref) {
             if ($ref eq 'SCALAR') {
-                $string .= $ddp->parse(\\$array_ref->[$idx]);
+                $string .= $ddp->parse(\$array_ref->[$idx]);
             }
             elsif ($ref eq 'REF') {
                 $string .= $ddp->parse(\$array_ref->[$idx]);
@@ -90,10 +98,7 @@ filter 'ARRAY' => sub {
             if $idx < $#{$array_ref} || $ddp->end_separator;
 
         # we're finished with "var[x]", turn it back to "var":
-        my $size = 2 + length($idx); # [10], [100], etc
-        my $name = $ddp->current_name;
-        substr $name, -$size, $size, '';
-        $ddp->current_name( $name );
+        $ddp->current_name( $original_varname );
     }
     $ddp->outdent;
     $ddp->{_array_padding} -= $local_padding if $has_index;
