@@ -22,7 +22,7 @@ filter '-class' => sub {
     # if the object overloads stringification, use it!
     # except for PDF::API2 which has a destructive stringify()
     if ($ddp->class->stringify && $class_name ne 'PDF::API2') {
-        my $str = _get_stringification($object, $class_name);
+        my $str = _get_stringification($ddp, $object, $class_name);
         return $ddp->maybe_colorize("$str ($class_name)", 'class')
             if defined $str;
     }
@@ -31,7 +31,10 @@ filter '-class' => sub {
     my $show_reftype = $ddp->class->show_reftype;
     my $show_internals = $ddp->class->internals;
     my $reftype;
-    $reftype = Scalar::Util::reftype($object) if $show_reftype || $show_internals;
+    if ($show_reftype || $show_internals) {
+        $reftype = Scalar::Util::reftype($object);
+        $reftype = 'Regexp' if $reftype eq 'REGEXP';
+    }
 
     $ddp->{_class_depth}++;
     my $string = $ddp->maybe_colorize( $class_name, 'class' );
@@ -49,25 +52,109 @@ filter '-class' => sub {
 
         my @superclasses = Data::Printer::Common::_get_superclasses_for($class_name);
         if (@superclasses && $ddp->class->parents) {
-            $string .= $ddp->newline . 'Parents       '
+            $string .= $ddp->newline . 'parents: '
                     . join(', ', map $ddp->maybe_colorize($_, 'class'), @superclasses)
                     ;
         }
+        my (%roles, %attributes);
+        if ($INC{'Role/Tiny.pm'} && exists $Role::Tiny::APPLIED_TO{$class_name}) {
+            %roles = %{ $Role::Tiny::APPLIED_TO{$class_name} };
+        }
+        my $is_moose = 0;
+
+        foreach my $parent (@$linear_ISA) {
+            if ($parent eq 'Moo::Object') {
+                Data::Printer::Common::_tryme(sub {
+                    my $moo_maker = 'Moo'->_constructor_maker_for($class_name);
+                    if (defined $moo_maker) {
+                        %attributes = %{ $moo_maker->all_attribute_specs };
+                    }
+                });
+                last;
+            }
+            elsif ($parent eq 'Moose::Object') {
+                Data::Printer::Common::_tryme(sub {
+                    my $class_meta = $class_name->meta;
+                    $is_moose = 1;
+                    %attributes = map {
+                        $_->name => {
+                            index => $_->insertion_order,
+                            init_arg => $_->init_arg,
+                            is => (defined $_->writer ? 'rw' : 'ro'),
+                            reader => $_->reader,
+                            required => $_->is_required,
+                        }
+                    } $class_meta->get_all_attributes();
+                    foreach my $role ($class_meta->calculate_all_roles()) {
+                        $roles{ $role->name } = 1;
+                    }
+                });
+                last;
+            }
+            elsif ($parent eq 'Object::Pad::UNIVERSAL') {
+                Data::Printer::Common::_tryme(sub {
+                    my $meta = Object::Pad::MOP::Class->for_class( $class_name );
+                    %attributes = map {
+                        $_->name . $_->value($class_name) => {
+                        }
+                    } $meta->fields;
+                    %roles = map { $_->name => 1 } $meta->direct_roles;
+                });
+            }
+        }
+        if ($ddp->class->show_methods ne 'none') {
+            if (my @role_list = keys %roles) {
+                @role_list = Data::Printer::Common::_nsort(@role_list)
+                    if @role_list && $ddp->class->sort_methods;
+                $string .= $ddp->newline . 'roles (' . scalar(@role_list) . '): '
+                        . join(', ' => map $ddp->maybe_colorize($_, 'class'), @role_list)
+                        ;
+            }
+
+            if (my @attr_list = keys %attributes) {
+                @attr_list = Data::Printer::Common::_nsort(@attr_list)
+                    if @attr_list && $ddp->class->sort_methods;
+                $string .= $ddp->newline . 'attributes (' . scalar(@attr_list) . '): '
+                        . join(', ' => map $ddp->maybe_colorize($_, 'method'), @attr_list)
+                        ;
+            }
+        }
 
         my $show_linear_isa = $ddp->class->linear_isa && (
              ($ddp->class->linear_isa eq 'auto' and @superclasses > 1)
           or ($ddp->class->linear_isa ne 'auto')
         );
-        my $show_methods = $ddp->class->show_methods && $ddp->class->show_methods ne 'none';
 
         if ($show_linear_isa && @$linear_ISA) {
-            $string .= $ddp->newline . 'Linear @ISA   '
+            $string .= $ddp->newline . 'linear @ISA: '
                     . join(', ' => map $ddp->maybe_colorize($_, 'class'), @$linear_ISA)
                     ;
         }
 
-        if ($show_methods) {
-            $string .= _show_methods($class_name, $linear_ISA, $ddp);
+        if ($ddp->class->show_methods ne 'none') {
+            $string .= _show_methods($class_name, $linear_ISA, \%attributes, $ddp);
+            if ($is_moose && $ddp->class->show_wrapped) {
+                my $modified = '';
+                my $modified_count = 0;
+                $ddp->indent;
+                for my $method ($class_name->meta->get_all_methods) {
+                    if (ref $method eq 'Class::MOP::Method::Wrapped') {
+                        foreach my $kind (qw(before around after)) {
+                            my $getter_method = $kind . '_modifiers';
+                            if (my @modlist = $method->$getter_method) {
+                                $modified .= $ddp->newline . $kind . ' ' . $method->name . ': '
+                                          . (@modlist > 1 ? $ddp->parse(\@modlist) : $ddp->parse($modlist[0]));
+                                $modified_count++;
+                            }
+                        }
+                    }
+                }
+                $ddp->outdent;
+                if ($modified_count) {
+                    $string .= $ddp->newline . 'method modifiers (' . $modified_count . '):'
+                            . $modified;
+                }
+            }
         }
 
         if ($ddp->class->show_overloads) {
@@ -92,6 +179,7 @@ filter '-class' => sub {
     if ($ddp->show_tied and my $tie = ref tied $object) {
         $string .= " (tied to $tie)";
     }
+
     return $string;
 };
 
@@ -100,7 +188,7 @@ filter '-class' => sub {
 #######################################
 
 sub _get_stringification {
-    my ($object, $class_name) = @_;
+    my ($ddp, $object, $class_name) = @_;
     require overload;
     if (overload::Overloaded($object)
         && (overload::Method($object, q(""))
@@ -111,6 +199,7 @@ sub _get_stringification {
         my $error = Data::Printer::Common::_tryme(sub { $string = '' . $object });
         if ($error) {
             Data::Printer::Common::_warn(
+                $ddp,
                 "string/number overload error for object $class_name: $error"
             );
         }
@@ -124,6 +213,7 @@ sub _get_stringification {
             my $error = Data::Printer::Common::_tryme(sub { $string = $object->$method });
             if ($error) {
                 Data::Printer::Common::_warn(
+                    $ddp,
                     "error stringifying object $class_name with $method\(\): $error"
                 );
             }
@@ -145,7 +235,7 @@ sub _get_overloads {
 }
 
 sub _show_methods {
-    my ($class_name, $linear_ISA, $ddp) = @_;
+    my ($class_name, $linear_ISA, $attributes, $ddp) = @_;
 
     my %methods = ( public => {}, private => {} );
     my @all_methods = map _methods_of(
@@ -156,6 +246,7 @@ sub _show_methods {
     my %seen_method_name;
     foreach my $method (@all_methods) {
         my ($package_string, $method_string) = @$method;
+        next if exists $attributes->{$method_string};
         next if $seen_method_name{$method_string}++;
         next if $method_string eq '__ANON__'; # anonymous subs don't matter here.
         my $type = substr($method_string, 0, 1) eq '_' ? 'private' : 'public';
@@ -176,7 +267,7 @@ sub _show_methods {
         if ($ddp->class->format_inheritance eq 'string') {
             my @method_list = keys %{$methods{$type}};
             @method_list = Data::Printer::Common::_nsort(@method_list)
-                if $ddp->class->sort_methods && @method_list;
+                if @method_list && $ddp->class->sort_methods;
 
             $string .= $ddp->newline . "$type methods (" . scalar(@method_list) . ')';
             if (@method_list) {
@@ -208,7 +299,7 @@ sub _show_methods {
 
             # then we print them, starting with our own methods:
             @base_methods = Data::Printer::Common::_nsort(@base_methods)
-                if $ddp->class->sort_methods && @base_methods;
+                if @base_methods && $ddp->class->sort_methods;
 
             $string .= $ddp->newline . "$type methods ($total_methods)"
                     . ($total_methods ? ':' : '')
@@ -222,8 +313,7 @@ sub _show_methods {
                 $string .= (keys %lined_methods ? $ddp->newline : ' ') . $base_string;
                 $ddp->outdent;
             }
-            foreach my $pkg (@$linear_ISA) {
-                next unless exists $lined_methods{$pkg};
+            foreach my $pkg (sort keys %lined_methods) {
                 $ddp->indent;
                 $string .= $ddp->newline . "$pkg:";
                 @{$lined_methods{$pkg}} = Data::Printer::Common::_nsort(@{$lined_methods{$pkg}})
